#include "logworker.h"
#include <iostream>
#include <unistd.h>
#include <sys/stat.h>

LogWorker::LogWorker(QObject *parent) : QObject(parent), conn(NULL)
{
}

LogWorker::~LogWorker()
{
    disconnectBackend();
}

/**
 * @brief CrazyflieWorker::connectBackend
 * Command the worker to connect to the backend
 * will set the private class struct, conn, once done.
 * The connected() signal is emitted once complete.
 */
void LogWorker::connectBackend()
{
    if (conn == NULL) {
        conn = ucart_backendConnect();
        emit (connected());
    } else {
        qInfo() << "Attempted to connect crazyflieworker when already connected!";
    }
    
}

/**
 * @brief CrazyflieWorker::disconnectBackend
 * Command the worker to disconnect from the backend,
 * sets the conn struct to null and emits the
 * disconnected() signal once complete
 */
void LogWorker::disconnectBackend()
{
    if (conn) {
         ucart_backendDisconnect(conn);
         conn = NULL;
         emit (disconnected());
    }
}